#include "cieter_dynarray.h"

struct dyn_array {
	char **array;
	size_t size;
	size_t capacity;
};

CieterDynArray *cieter_dynarray_init(size_t initial_capacity) {
	CieterDynArray *da = malloc(sizeof(CieterDynArray));
	da->size = 0;
	da->capacity = initial_capacity;

	return da;
}

void cieter_dynarray_add(CieterDynArray *da, const char *s) {
	// An empty dynarray does not have an allocated internal array yet
	if (da->size == 0) {
		da->array = malloc(sizeof(char*) * da->capacity);
	}
	// Double array size if it's full
	else if (da->size == da->capacity) {
		da->array = realloc(da->array, da->capacity * 2);
		da->capacity *= 2;
	}

	da->array[da->size] = strdup(s);
	da->size++;
}

void cieter_dynarray_free(CieterDynArray *da) {
	if (da == NULL) {
		return;
	}

	if (da->array != NULL) {
		for (size_t i = 0; i < da->size; i++) {
			free(da->array[i]);
		}

		free(da->array);
	}

	free(da);
}

char **cieter_dynarray_convert(CieterDynArray *da) {
	char **array = da->array;

	da->array = NULL;
	cieter_dynarray_free(da);

	return array;
}
