#include "cieter_package.h"

static char *ignored_names[5] = {
	".BUILDINFO",
	".INSTALL",
	".MTREE",
	".PKGINFO",
	".CHANGELOG"
};
static int ignored_words_len = sizeof(ignored_names) / sizeof(char *);

CieterPkg *package_init() {
	return calloc(sizeof(CieterPkgInfo), 1);
}

CieterPkg *package_read_archive(const char *pkg_path) {
	struct archive *a = archive_read_new();
	struct archive_entry *entry = archive_entry_new();

	// These three are the most commonly used compression methods
	archive_read_support_filter_zstd(a);
	archive_read_support_filter_gzip(a);
	archive_read_support_filter_xz(a);

	// Contents should always be a tarball
	archive_read_support_format_tar(a);

	// TODO where does this 10240 come from?
	int r = archive_read_open_filename(a, pkg_path, 10240);

	// Exit early if we weren't able to successfully open the archive for reading
	if (r != ARCHIVE_OK) {
		return NULL;
	}

	int compression_code = archive_filter_code(a, 0);
	const char *path_name;

	CieterPkgInfo *pkg_info;
	CieterDynArray *files = cieter_dynarray_init(16);

	while (archive_read_next_header(a, &entry) == ARCHIVE_OK) {
		path_name = archive_entry_pathname(entry);

		bool ignore = false;

		for (size_t i = 0; i < ignored_words_len; i++) {
			if (strcmp(path_name, ignored_names[i]) == 0) {
				ignore = true;
				break;
			}
		}

		if (!ignore) {
			cieter_dynarray_add(files, path_name);
		}

		if (strcmp(path_name, ".PKGINFO") == 0) {
			// Read data of file into memory buffer
			int size = archive_entry_size(entry);
			char *buf = malloc(size);
			archive_read_data(a, buf, size);

			// Parse package info string into a struct
			pkg_info = package_info_init();
			package_info_parse(pkg_info, buf);

			free(buf);
		} else {
			archive_read_data_skip(a);
		}
	}

	// Get size of file
	struct stat stats;

	if (stat(pkg_path, &stats) != 0) {
		return NULL;
	}

	pkg_info->csize = stats.st_size;

	archive_read_free(a);
	archive_entry_free(entry);

	// Create final return value
	CieterPkg *pkg = package_init();
	pkg->path = strdup(pkg_path);
	pkg->info = pkg_info;
	pkg->files = files;
	pkg->compression = compression_code;

	return pkg;
}

char *package_to_description(CieterPkg *pkg) {

}
